@@ -362,7 +362,7 @@ static void
 dvdnavsrc_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec) 
 {
   DVDNavSrc *src;
-  char *title_string;
+  const char *title_string;
 
   /* it's not null if we got it, but it might not be ours */
   g_return_if_fail (GST_IS_DVDNAVSRC (object));
@@ -591,45 +591,51 @@ dvdnavsrc_button_pressed (DVDNavSrc *src, int button)
 static void
 dvdnavsrc_pointer_select (DVDNavSrc *src, int x, int y)
 {
-  dvdnav_mouse_select(src->dvdnav, x, y);
+  dvdnav_mouse_select(src->dvdnav,
+                      dvdnav_get_current_nav_pci(src->dvdnav),
+                      x, y);
 }
 
 static void
 dvdnavsrc_pointer_activate (DVDNavSrc *src, int x, int y)
 {
-  dvdnav_mouse_activate(src->dvdnav, x, y);
+  dvdnav_mouse_activate(src->dvdnav,
+                        dvdnav_get_current_nav_pci(src->dvdnav),
+                        x, y);
 }
 
 static void
 dvdnavsrc_user_op (DVDNavSrc *src, int op)
 {
+  pci_t *pci = dvdnav_get_current_nav_pci(src->dvdnav);
+
   fprintf (stderr, "user_op %d\n", op);
   /* Magic user_op ids */
   switch (op) {
     case 0: /* None */
       break;
     case 1: /* Upper */
-      if (dvdnav_upper_button_select(src->dvdnav) != DVDNAV_STATUS_OK) {
+      if (dvdnav_upper_button_select(src->dvdnav, pci) != DVDNAV_STATUS_OK) {
         goto naverr;
       }
       break;
     case 2: /* Lower */
-      if (dvdnav_lower_button_select(src->dvdnav) != DVDNAV_STATUS_OK) {
+      if (dvdnav_lower_button_select(src->dvdnav, pci) != DVDNAV_STATUS_OK) {
         goto naverr;
       }
       break;
     case 3: /* Left */
-      if (dvdnav_left_button_select(src->dvdnav) != DVDNAV_STATUS_OK) {
+      if (dvdnav_left_button_select(src->dvdnav, pci) != DVDNAV_STATUS_OK) {
         goto naverr;
       }
       break;
     case 4: /* Right */
-      if (dvdnav_right_button_select(src->dvdnav) != DVDNAV_STATUS_OK) {
+      if (dvdnav_right_button_select(src->dvdnav, pci) != DVDNAV_STATUS_OK) {
         goto naverr;
       }
       break;
     case 5: /* Activate */
-      if (dvdnav_button_activate(src->dvdnav) != DVDNAV_STATUS_OK) {
+      if (dvdnav_button_activate(src->dvdnav, pci) != DVDNAV_STATUS_OK) {
         goto naverr;
       }
       break;
@@ -706,8 +712,8 @@ dvdnav_get_event_name(int event)
     case DVDNAV_STOP: return "DVDNAV_STOP"; break;
     case DVDNAV_HIGHLIGHT: return "DVDNAV_HIGHLIGHT"; break;
     case DVDNAV_SPU_CLUT_CHANGE: return "DVDNAV_SPU_CLUT_CHANGE"; break;
-    case DVDNAV_SEEK_DONE: return "DVDNAV_SEEK_DONE"; break;
     case DVDNAV_HOP_CHANNEL: return "DVDNAV_HOP_CHANNEL"; break;
+    case DVDNAV_WAIT: return "DVDNAV_WAIT"; break;
   }
   return "UNKNOWN";
 }
@@ -770,27 +776,28 @@ dvdnavsrc_print_event (DVDNavSrc *src, guint8 *data, int event, int len)
     case DVDNAV_CELL_CHANGE:
       {
         dvdnav_cell_change_event_t *event = (dvdnav_cell_change_event_t *)data;
-        fprintf (stderr, "  old_cell: %p\n", event->old_cell);
-        fprintf (stderr, "  new_cell: %p\n", event->new_cell);
+        /*fprintf (stderr, "  old_cell: %p\n", event->old_cell);*/
+        /*fprintf (stderr, "  new_cell: %p\n", event->new_cell);*/
       }
       break;
     case DVDNAV_NAV_PACKET:
       {
+/*
         dvdnav_nav_packet_event_t *event = (dvdnav_nav_packet_event_t *)data;
         pci_t *pci;
         dsi_t *dsi;
-        /*
+
         pci = event->pci;
         dsi = event->dsi;
-        */
+
         pci = dvdnav_get_current_nav_pci(src->dvdnav);
         dsi = dvdnav_get_current_nav_dsi(src->dvdnav);
         fprintf (stderr, "  pci: %p\n", event->pci);
         fprintf (stderr, "  dsi: %p\n", event->dsi);
-        /*
+
         navPrint_PCI(pci);
         navPrint_DSI(dsi);
-        */
+*/
       }
       break;
     case DVDNAV_STOP:
@@ -811,10 +818,10 @@ dvdnavsrc_print_event (DVDNavSrc *src, guint8 *data, int event, int len)
       break;
     case DVDNAV_SPU_CLUT_CHANGE:
       break;
-    case DVDNAV_SEEK_DONE:
-      break;
     case DVDNAV_HOP_CHANNEL:
       break;
+    case DVDNAV_WAIT:
+      break;
     default:
       fprintf (stderr, "  event id: %d\n", event);
       break;
@@ -906,8 +913,8 @@ dvdnavsrc_get (GstPad *pad)
       case DVDNAV_AUDIO_STREAM_CHANGE:
       case DVDNAV_HIGHLIGHT:
       case DVDNAV_SPU_CLUT_CHANGE:
-      case DVDNAV_SEEK_DONE:
       case DVDNAV_HOP_CHANNEL:
+      case DVDNAV_WAIT:
       default:
         dvdnavsrc_print_event (src, data, event, len);
         break;
