@@ -21,7 +21,6 @@
 
 #include <inttypes.h>
 
-#include <mpeg2dec/mm_accel.h>
 #include "gstmpeg2dec.h"
 
 /* elementfactory information */
@@ -31,8 +30,8 @@ static GstElementDetails gst_mpeg2dec_details = {
   "GPL",
   "Uses libmpeg2 to decode MPEG video streams",
   VERSION,
-  "Wim Taymans <wim.taymans@chello.be>, "
-  "(C) 2002",
+  "Wim Taymans <wim.taymans@chello.be>",
+  "(C) 2002"
 };
 
 /* Mpeg2dec signals and args */
@@ -43,7 +42,7 @@ enum {
 
 enum {
   ARG_0,
-  ARG_FRAME_RATE,
+  ARG_STREAMINFO,
   /* FILL ME */
 };
 
@@ -71,6 +70,17 @@ GST_PAD_TEMPLATE_FACTORY (src_template_factory,
   )
 );
 
+GST_PAD_TEMPLATE_FACTORY (user_data_template_factory,
+  "user_data",
+  GST_PAD_SRC,
+  GST_PAD_ALWAYS,
+  GST_CAPS_NEW (
+    "mpeg2dec_user_data",
+    "application/octet-stream",
+    NULL
+  )
+);
+
 GST_PAD_TEMPLATE_FACTORY (sink_template_factory,
   "sink",
   GST_PAD_SINK,
@@ -92,17 +102,17 @@ static void		gst_mpeg2dec_set_property	(GObject *object, guint prop_id,
 							 const GValue *value, GParamSpec *pspec);
 static void		gst_mpeg2dec_get_property	(GObject *object, guint prop_id, 
 							 GValue *value, GParamSpec *pspec);
-static void 		gst_mpeg2dec_set_cache 		(GstElement *element, GstCache *cache);
-static GstCache*	gst_mpeg2dec_get_cache 		(GstElement *element);
+static void 		gst_mpeg2dec_set_index 		(GstElement *element, GstIndex *index);
+static GstIndex*	gst_mpeg2dec_get_index 		(GstElement *element);
 
 static const GstFormat*
 			gst_mpeg2dec_get_src_formats 	(GstPad *pad);
 static const GstEventMask*
 			gst_mpeg2dec_get_src_event_masks (GstPad *pad);
 static gboolean 	gst_mpeg2dec_src_event       	(GstPad *pad, GstEvent *event);
-static const GstPadQueryType*
+static const GstQueryType*
 			gst_mpeg2dec_get_src_query_types (GstPad *pad);
-static gboolean 	gst_mpeg2dec_src_query 		(GstPad *pad, GstPadQueryType type,
+static gboolean 	gst_mpeg2dec_src_query 		(GstPad *pad, GstQueryType type,
 			       				 GstFormat *format, gint64 *value);
 
 static const GstFormat*
@@ -153,17 +163,17 @@ gst_mpeg2dec_class_init(GstMpeg2decClass *klass)
 
   parent_class = g_type_class_ref(GST_TYPE_ELEMENT);
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FRAME_RATE,
-    g_param_spec_float ("frame_rate","frame_rate","frame_rate",
-                        0.0, 1000.0, 0.0, G_PARAM_READABLE)); 
+  g_object_class_install_property (gobject_class, ARG_STREAMINFO,
+    g_param_spec_boxed ("streaminfo", "Streaminfo", "Streaminfo",
+                        GST_TYPE_CAPS, G_PARAM_READABLE));
   
   gobject_class->set_property 	= gst_mpeg2dec_set_property;
   gobject_class->get_property 	= gst_mpeg2dec_get_property;
   gobject_class->dispose 	= gst_mpeg2dec_dispose;
 
   gstelement_class->change_state = gst_mpeg2dec_change_state;
-  gstelement_class->set_cache 	 = gst_mpeg2dec_set_cache;
-  gstelement_class->get_cache 	 = gst_mpeg2dec_get_cache;
+  gstelement_class->set_index 	 = gst_mpeg2dec_set_index;
+  gstelement_class->get_index 	 = gst_mpeg2dec_get_index;
 }
 
 static void
@@ -187,43 +197,67 @@ gst_mpeg2dec_init (GstMpeg2dec *mpeg2dec)
   gst_pad_set_query_function (mpeg2dec->srcpad, GST_DEBUG_FUNCPTR (gst_mpeg2dec_src_query));
   gst_pad_set_convert_function (mpeg2dec->srcpad, GST_DEBUG_FUNCPTR (gst_mpeg2dec_convert_src));
 
-  /* initialize the mpeg2dec decoder state */
-  mpeg2dec->decoder = mpeg2_init (mm_accel());
+  mpeg2dec->userdatapad = gst_pad_new_from_template (
+		  GST_PAD_TEMPLATE_GET (user_data_template_factory), "user_data");
+  gst_element_add_pad (GST_ELEMENT (mpeg2dec), mpeg2dec->userdatapad);
+
+  /* initialize the mpeg2dec acceleration */
+  mpeg2_accel (MPEG2_ACCEL_DETECT);
+  mpeg2dec->closed = TRUE;
+  mpeg2dec->have_fbuf = FALSE;
 
   GST_FLAG_SET (GST_ELEMENT (mpeg2dec), GST_ELEMENT_EVENT_AWARE);
 }
 
+static void
+gst_mpeg2dec_close_decoder (GstMpeg2dec *mpeg2dec)
+{
+  if (!mpeg2dec->closed) {
+    mpeg2_close (mpeg2dec->decoder);
+    mpeg2dec->closed = TRUE;
+    mpeg2dec->decoder = NULL;
+  }
+}
+
+static void
+gst_mpeg2dec_open_decoder (GstMpeg2dec *mpeg2dec)
+{
+  gst_mpeg2dec_close_decoder (mpeg2dec);
+  mpeg2dec->decoder = mpeg2_init ();
+  mpeg2dec->closed = FALSE;
+  mpeg2dec->have_fbuf = FALSE;
+}
+
 static void
 gst_mpeg2dec_dispose (GObject *object)
 {
   GstMpeg2dec *mpeg2dec = GST_MPEG2DEC (object);
 
-  if (!mpeg2dec->closed) 
-    mpeg2_close (mpeg2dec->decoder);
+  gst_mpeg2dec_close_decoder (mpeg2dec);
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
-gst_mpeg2dec_set_cache (GstElement *element, GstCache *cache)
+gst_mpeg2dec_set_index (GstElement *element, GstIndex *index)
 {
   GstMpeg2dec *mpeg2dec = GST_MPEG2DEC (element);
   
-  mpeg2dec->cache = cache;
+  mpeg2dec->index = index;
 
-  gst_cache_get_writer_id (cache, GST_OBJECT (element), &mpeg2dec->cache_id);
+  gst_index_get_writer_id (index, GST_OBJECT (element), &mpeg2dec->index_id);
 }
 
-static GstCache*
-gst_mpeg2dec_get_cache (GstElement *element)
+static GstIndex*
+gst_mpeg2dec_get_index (GstElement *element)
 {
   GstMpeg2dec *mpeg2dec = GST_MPEG2DEC (element);
 
-  return mpeg2dec->cache;
+  return mpeg2dec->index;
 }
 
-static gboolean
-gst_mpeg2dec_alloc_buffer (GstMpeg2dec *mpeg2dec, const mpeg2_info_t *info)
+static GstBuffer*
+gst_mpeg2dec_alloc_buffer (GstMpeg2dec *mpeg2dec, const mpeg2_info_t *info, gint64 offset)
 {
   GstBuffer *outbuf = NULL;
   gint size = mpeg2dec->width * mpeg2dec->height;
@@ -254,15 +288,17 @@ gst_mpeg2dec_alloc_buffer (GstMpeg2dec *mpeg2dec, const mpeg2_info_t *info)
 
   picture = info->current_picture;
 
-  if (picture && (picture->flags & PIC_MASK_CODING_TYPE) == PIC_FLAG_CODING_TYPE_I)
-  {
+  if (picture && (picture->flags & PIC_MASK_CODING_TYPE) == PIC_FLAG_CODING_TYPE_I) {
     GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_KEY_UNIT);
   }
   else {
     GST_BUFFER_FLAG_UNSET (outbuf, GST_BUFFER_KEY_UNIT);
   }
+  /* we store the original byteoffset of this picture in the stream here
+   * because we need it for indexing */
+  GST_BUFFER_OFFSET (outbuf) = offset;
 
-  return TRUE;
+  return outbuf;
 }
 
 static gboolean
@@ -322,6 +358,54 @@ gst_mpeg2dec_negotiate_format (GstMpeg2dec *mpeg2dec)
   return TRUE;
 }
 
+static void
+update_streaminfo (GstMpeg2dec *mpeg2dec)
+{
+  GstCaps *caps;
+  GstProps *props;
+  GstPropsEntry *entry;
+  const mpeg2_info_t *info;
+
+  info = mpeg2_info (mpeg2dec->decoder);
+
+  props = gst_props_empty_new ();
+
+  entry = gst_props_entry_new ("framerate", GST_PROPS_FLOAT (GST_SECOND/(float)mpeg2dec->frame_period));
+  gst_props_add_entry (props, entry);
+  entry = gst_props_entry_new ("bitrate", GST_PROPS_INT (info->sequence->byte_rate * 8));
+  gst_props_add_entry (props, entry);
+
+  caps = gst_caps_new ("mpeg2dec_streaminfo",
+                       "application/x-gst-streaminfo",
+                        props);
+
+  if (mpeg2dec->streaminfo)
+    gst_caps_unref (mpeg2dec->streaminfo);
+
+  mpeg2dec->streaminfo = caps;
+  g_object_notify (G_OBJECT (mpeg2dec), "streaminfo");
+}
+
+static void
+gst_mpeg2dec_flush_decoder (GstMpeg2dec *mpeg2dec)
+{
+  gint state;
+
+  if (mpeg2dec->decoder) {
+    const mpeg2_info_t *info = mpeg2_info (mpeg2dec->decoder);
+
+    do {
+      state = mpeg2_parse (mpeg2dec->decoder);
+      if (state == STATE_END) {
+	if (info->discard_fbuf && info->discard_fbuf->id) {
+	  gst_buffer_unref ((GstBuffer *)info->discard_fbuf->id);
+	}
+      }
+    } 
+    while (state != -1);
+  }
+}
+
 static void
 gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
 {
@@ -339,20 +423,16 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
     switch (GST_EVENT_TYPE (event)) {
       case GST_EVENT_DISCONTINUOUS:
       {
-	//gint64 value = GST_EVENT_DISCONT_OFFSET (event, 0).value;
-	//mpeg2dec->decoder->is_sequence_needed = 1;
-	GST_DEBUG (GST_CAT_EVENT, "discont\n"); 
-        mpeg2dec->first = TRUE;
-        mpeg2dec->last_PTS = -1;
+	GST_DEBUG (GST_CAT_EVENT, "discont"); 
         mpeg2dec->next_time = 0;
-        mpeg2dec->discont_pending = TRUE;
+        mpeg2dec->discont_state = MPEG2DEC_DISC_NEW_PICTURE;
+	gst_mpeg2dec_flush_decoder (mpeg2dec);
         gst_pad_event_default (pad, event);
 	return;
       }
       case GST_EVENT_EOS:
-        if (!mpeg2dec->closed) {
-          mpeg2_close (mpeg2dec->decoder); 
-	  mpeg2dec->closed = TRUE;
+	if (mpeg2dec->index && mpeg2dec->closed) {
+	  gst_index_commit (mpeg2dec->index, mpeg2dec->index_id);
         }
       default:
         gst_pad_event_default (pad, event);
@@ -379,10 +459,16 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
     GST_DEBUG (GST_CAT_CLOCK, "no pts");
   }
 
+  GST_DEBUG (0, "calling _buffer");
   mpeg2_buffer (mpeg2dec->decoder, data, end);
+  GST_DEBUG (0, "calling _buffer done");
 
   while (!done) {
+    gboolean slice = FALSE;
+
+    GST_DEBUG (0, "calling parse");
     state = mpeg2_parse (mpeg2dec->decoder);
+    GST_DEBUG (0, "parse state %d", state);
     switch (state) {
       case STATE_SEQUENCE:
       {
@@ -398,98 +484,135 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
 
 	if (!gst_mpeg2dec_negotiate_format (mpeg2dec)) {
           gst_element_error (GST_ELEMENT (mpeg2dec), "could not negotiate format");
-	  return;
+	  goto exit;
 	}
 
-	gst_mpeg2dec_alloc_buffer (mpeg2dec, info);
+	update_streaminfo (mpeg2dec);
+
+	if (!mpeg2dec->have_fbuf) {
+	  /* alloc 3 buffers */
+	  gst_mpeg2dec_alloc_buffer (mpeg2dec, info, GST_BUFFER_OFFSET (buf));
+	  gst_mpeg2dec_alloc_buffer (mpeg2dec, info, GST_BUFFER_OFFSET (buf));
+	  gst_mpeg2dec_alloc_buffer (mpeg2dec, info, GST_BUFFER_OFFSET (buf));
+	  mpeg2dec->have_fbuf = TRUE;
+	}
+
+	mpeg2dec->need_sequence = FALSE;
+	if (mpeg2dec->pending_event) {
+	  done = GST_EVENT_SEEK_FLAGS (mpeg2dec->pending_event) & GST_SEEK_FLAG_FLUSH;
+	  
+          gst_mpeg2dec_src_event (mpeg2dec->srcpad, mpeg2dec->pending_event);
+	  mpeg2dec->pending_event = NULL;
+	}
         break;
       }
       case STATE_SEQUENCE_REPEATED:
 	GST_DEBUG (0, "sequence repeated");
-        break;
       case STATE_GOP:
         break;
       case STATE_PICTURE:
       {
 	gboolean key_frame = FALSE;
+	GstBuffer *outbuf;
 
-	
-        if (info->current_picture)
+        if (info->current_picture) {
 	  key_frame = (info->current_picture->flags & PIC_MASK_CODING_TYPE) == PIC_FLAG_CODING_TYPE_I;
-
-	gst_mpeg2dec_alloc_buffer (mpeg2dec, info);
-
-        if (key_frame && mpeg2dec->discont_pending) {
-	  mpeg2dec->discont_pending = FALSE;
-	  mpeg2dec->first = TRUE;
-          if (pts != -1 && mpeg2dec->last_PTS == -1) {
-            mpeg2dec->last_PTS = pts;
-            mpeg2dec->next_time = pts;
-          }
 	}
+	outbuf = gst_mpeg2dec_alloc_buffer (mpeg2dec, info, GST_BUFFER_OFFSET (buf));
+
+	GST_DEBUG (0, "picture %d, %p, %lld, %lld", 
+			key_frame, outbuf, GST_BUFFER_OFFSET (outbuf), pts);
+
+	if (mpeg2dec->discont_state == MPEG2DEC_DISC_NEW_PICTURE && key_frame)
+	  mpeg2dec->discont_state = MPEG2DEC_DISC_NEW_KEYFRAME;
+
+	if (!GST_PAD_IS_USABLE (mpeg2dec->srcpad))
+	  mpeg2_skip (mpeg2dec->decoder, 1);
+	else
+	  mpeg2_skip (mpeg2dec->decoder, 0);
 
-	if (mpeg2dec->cache && pts != GST_CLOCK_TIME_NONE) {
-          gst_cache_add_association (mpeg2dec->cache, mpeg2dec->cache_id,
-	                             (key_frame ? GST_ACCOCIATION_FLAG_KEY_UNIT : 0),
-	                             GST_FORMAT_BYTES, GST_BUFFER_OFFSET (buf),
-	                             GST_FORMAT_TIME, pts, 0);
-	}
         break;
       }
       case STATE_SLICE_1ST:
 	GST_DEBUG (0, "slice 1st");
         break;
       case STATE_PICTURE_2ND:
-	GST_DEBUG (0, "picture second\n");
+	GST_DEBUG (0, "picture second");
         break;
       case STATE_SLICE:
+	slice = TRUE;
       case STATE_END:
       {
 	GstBuffer *outbuf = NULL;
+	gboolean skip = FALSE;
+
+	if (!slice) {
+	  mpeg2dec->need_sequence = TRUE;
+	}
+	GST_DEBUG (0, "picture end %p %p %p %p", info->display_fbuf, info->display_picture, info->current_picture,
+			(info->display_fbuf ? info->display_fbuf->id : NULL));
 
 	if (info->display_fbuf && info->display_fbuf->id) {
           const picture_t *picture;
+	  gboolean key_frame = FALSE;
 	  
 	  outbuf = (GstBuffer *) info->display_fbuf->id;
 	  picture = info->display_picture;
 
+	  key_frame = (picture->flags & PIC_MASK_CODING_TYPE) == PIC_FLAG_CODING_TYPE_I;
+	  GST_DEBUG (0, "picture keyfame %d", key_frame);
+
+	  if (key_frame) {
+            GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_KEY_UNIT);
+	  }
+	  if (mpeg2dec->discont_state == MPEG2DEC_DISC_NEW_KEYFRAME && key_frame)
+	    mpeg2dec->discont_state = MPEG2DEC_DISC_NONE;
+
 	  if (picture->flags & PIC_FLAG_PTS) {
             GstClockTime time = MPEGTIME_TO_GSTTIME (picture->pts);
 
+	    GST_DEBUG (0, "picture had pts %lld", time);
             GST_BUFFER_TIMESTAMP (outbuf) = time;
 
             mpeg2dec->next_time = time;
 	  }
 	  else {
+	    GST_DEBUG (0, "picture didn't have pts using %lld", mpeg2dec->next_time);
             GST_BUFFER_TIMESTAMP (outbuf) = mpeg2dec->next_time;
 	  }
           mpeg2dec->next_time += (mpeg2dec->frame_period * picture->nb_fields) >> 1;
-	  
-	  GST_DEBUG (0, "picture: %s %s fields:%d ts:%lld", 
+
+
+	  GST_DEBUG (0, "picture: %s %s fields:%d off:%lld ts:%lld", 
 			  (picture->flags & PIC_FLAG_TOP_FIELD_FIRST ? "tff " : "    "),
 			  (picture->flags & PIC_FLAG_PROGRESSIVE_FRAME ? "prog" : "    "),
 			  picture->nb_fields, 
+	                  GST_BUFFER_OFFSET (outbuf),
 	                  GST_BUFFER_TIMESTAMP (outbuf));
 
-	  /*
-	  if (picture->flags & PIC_FLAG_SKIP ||
-	      mpeg2dec->discont_pending ||
-	      (mpeg2dec->first && !GST_BUFFER_FLAG_IS_SET (outbuf, GST_BUFFER_KEY_UNIT))) 
-	  {
-	  */
+	  if (mpeg2dec->index) {
+            gst_index_add_association (mpeg2dec->index, mpeg2dec->index_id,
+	                               (key_frame ? GST_ASSOCIATION_FLAG_KEY_UNIT : 0),
+	                               GST_FORMAT_BYTES, GST_BUFFER_OFFSET (outbuf),
+	                               GST_FORMAT_TIME, GST_BUFFER_TIMESTAMP (outbuf), 0);
+	  }
+
+	  
 	  if (picture->flags & PIC_FLAG_SKIP || 
-	      !GST_PAD_IS_USABLE (mpeg2dec->srcpad)) {
+	      !GST_PAD_IS_USABLE (mpeg2dec->srcpad) || 
+	      mpeg2dec->discont_state != MPEG2DEC_DISC_NONE ||
+	      mpeg2dec->next_time < mpeg2dec->segment_start ||
+	      skip) 
+	  {
 	    gst_buffer_unref (outbuf);
 	  }
 	  else {
-            mpeg2dec->first = FALSE;
 	    gst_pad_push (mpeg2dec->srcpad, outbuf);
 	  }
 	}
 	if (info->discard_fbuf && info->discard_fbuf->id) {
 	  gst_buffer_unref ((GstBuffer *)info->discard_fbuf->id);
 	}
-
         break;
       }
       /* need more data */
@@ -498,8 +621,10 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
 	break;
       /* error */
       case STATE_INVALID:
-	gst_element_error (GST_ELEMENT (mpeg2dec), "fatal error");
-	done = TRUE;
+	g_warning ("mpeg2dec: decoding error");
+	/* it looks like setting a new frame in libmpeg2 avoids a crash */
+	/* FIXME figure out how this screws up sync and buffer leakage */
+	gst_mpeg2dec_alloc_buffer (mpeg2dec, info, GST_BUFFER_OFFSET (buf));
         break;
       default:
 	g_warning ("%s: unhandled state %d, FIXME", 
@@ -507,7 +632,21 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
 			state);
         break;
     }
+
+    /*
+     * FIXME: should pass more information such as state the user data is from
+     */
+    if (info->user_data_len > 0) {
+      if (GST_PAD_IS_USABLE (mpeg2dec->userdatapad)) {
+        GstBuffer *udbuf = gst_buffer_new_and_alloc (info->user_data_len);
+
+        memcpy (GST_BUFFER_DATA (udbuf), info->user_data, info->user_data_len);
+
+        gst_pad_push (mpeg2dec->userdatapad, udbuf);
+      }
+    }
   }
+exit:
   gst_buffer_unref(buf);
 }
 
@@ -532,6 +671,9 @@ gst_mpeg2dec_convert_sink (GstPad *pad, GstFormat src_format, gint64 src_value,
 	      
   mpeg2dec = GST_MPEG2DEC (gst_pad_get_parent (pad));
 
+  if (mpeg2dec->decoder == NULL)
+    return FALSE;
+
   info = mpeg2_info (mpeg2dec->decoder);
 
   switch (src_format) {
@@ -590,6 +732,9 @@ gst_mpeg2dec_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
 	      
   mpeg2dec = GST_MPEG2DEC (gst_pad_get_parent (pad));
 
+  if (mpeg2dec->decoder == NULL)
+    return FALSE;
+
   info = mpeg2_info (mpeg2dec->decoder);
 
   switch (src_format) {
@@ -637,19 +782,19 @@ gst_mpeg2dec_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
   return res;
 }
 
-static const GstPadQueryType*
+static const GstQueryType*
 gst_mpeg2dec_get_src_query_types (GstPad *pad)
 {
-  static const GstPadQueryType types[] = {
-    GST_PAD_QUERY_TOTAL,
-    GST_PAD_QUERY_POSITION,
+  static const GstQueryType types[] = {
+    GST_QUERY_TOTAL,
+    GST_QUERY_POSITION,
     0
   };
   return types;
 }
 
 static gboolean 
-gst_mpeg2dec_src_query (GstPad *pad, GstPadQueryType type,
+gst_mpeg2dec_src_query (GstPad *pad, GstQueryType type,
 		        GstFormat *format, gint64 *value)
 {
   gboolean res = TRUE;
@@ -659,7 +804,7 @@ gst_mpeg2dec_src_query (GstPad *pad, GstPadQueryType type,
   mpeg2dec = GST_MPEG2DEC (gst_pad_get_parent (pad));
 
   switch (type) {
-    case GST_PAD_QUERY_TOTAL:
+    case GST_QUERY_TOTAL:
     {
       switch (*format) {
         case GST_FORMAT_DEFAULT:
@@ -682,7 +827,7 @@ gst_mpeg2dec_src_query (GstPad *pad, GstPadQueryType type,
 	    peer_format = *formats;
 	  
             /* do the probe */
-            if (gst_pad_query (GST_PAD_PEER (mpeg2dec->sinkpad), GST_PAD_QUERY_TOTAL,
+            if (gst_pad_query (GST_PAD_PEER (mpeg2dec->sinkpad), GST_QUERY_TOTAL,
 			       &peer_format, &peer_value)) 
 	    {
               GstFormat conv_format;
@@ -707,7 +852,7 @@ gst_mpeg2dec_src_query (GstPad *pad, GstPadQueryType type,
       }
       break;
     }
-    case GST_PAD_QUERY_POSITION:
+    case GST_QUERY_POSITION:
     {
       switch (*format) {
         case GST_FORMAT_DEFAULT:
@@ -740,6 +885,116 @@ gst_mpeg2dec_get_src_event_masks (GstPad *pad)
   return masks;
 }
 
+static gboolean 
+index_seek (GstPad *pad, GstEvent *event)
+{
+  GstIndexEntry *entry;
+  GstMpeg2dec *mpeg2dec;
+
+  mpeg2dec = GST_MPEG2DEC (gst_pad_get_parent (pad));
+  
+  entry = gst_index_get_assoc_entry (mpeg2dec->index, mpeg2dec->index_id,
+		             GST_INDEX_LOOKUP_BEFORE, GST_ASSOCIATION_FLAG_KEY_UNIT,
+			     GST_EVENT_SEEK_FORMAT (event),
+			     GST_EVENT_SEEK_OFFSET (event));
+
+  if (entry) {
+    const GstFormat *peer_formats, *try_formats;
+    /* since we know the exaxt byteoffset of the frame, make sure to seek on bytes first */
+    const GstFormat try_all_formats[] = 
+    { 
+	GST_FORMAT_BYTES,
+	GST_FORMAT_TIME,
+	0 
+    };
+
+    try_formats = try_all_formats;
+    peer_formats = gst_pad_get_formats (GST_PAD_PEER (mpeg2dec->sinkpad));
+
+    while (gst_formats_contains (peer_formats, *try_formats)) {
+      gint64 value;
+    
+      if (gst_index_entry_assoc_map (entry, *try_formats, &value)) {
+        GstEvent *seek_event;
+
+        /* lookup succeeded, create the seek */
+        seek_event = gst_event_new_seek (*try_formats | GST_SEEK_METHOD_SET | GST_SEEK_FLAG_FLUSH, value);
+        /* do the seekk */
+        if (gst_pad_send_event (GST_PAD_PEER (mpeg2dec->sinkpad), seek_event)) {
+          /* seek worked, we're done, loop will exit */
+	  gst_mpeg2dec_flush_decoder (mpeg2dec);
+	  mpeg2dec->segment_start = GST_EVENT_SEEK_OFFSET (event);
+          return TRUE;
+        }
+      }
+      try_formats++;
+    }
+  }
+  return FALSE;
+}
+
+
+static gboolean 
+normal_seek (GstPad *pad, GstEvent *event)
+{
+  gint64 src_offset;
+  gboolean flush;
+  GstFormat format;
+  const GstFormat *peer_formats;
+  gboolean res = FALSE;
+  GstMpeg2dec *mpeg2dec;
+
+  mpeg2dec = GST_MPEG2DEC (gst_pad_get_parent (pad));
+
+  format = GST_FORMAT_TIME;
+
+  /* first bring the src_format to TIME */
+  if (!gst_pad_convert (pad,
+            GST_EVENT_SEEK_FORMAT (event), GST_EVENT_SEEK_OFFSET (event),
+            &format, &src_offset))
+  {
+    /* didn't work, probably unsupported seek format then */
+    return res;
+  }
+
+  /* shave off the flush flag, we'll need it later */
+  flush = GST_EVENT_SEEK_FLAGS (event) & GST_SEEK_FLAG_FLUSH;
+
+  /* get our peer formats */
+  peer_formats = gst_pad_get_formats (GST_PAD_PEER (mpeg2dec->sinkpad));
+
+  /* while we did not exhaust our seek formats without result */
+  while (peer_formats && *peer_formats) {
+    gint64 desired_offset;
+
+    format = *peer_formats;
+
+    /* try to convert requested format to one we can seek with on the sinkpad */
+    if (gst_pad_convert (mpeg2dec->sinkpad, GST_FORMAT_TIME, src_offset, &format, &desired_offset))
+    {
+      GstEvent *seek_event;
+
+      /* conversion succeeded, create the seek */
+      seek_event = gst_event_new_seek (format | GST_SEEK_METHOD_SET | flush, desired_offset);
+      /* do the seekk */
+      if (gst_pad_send_event (GST_PAD_PEER (mpeg2dec->sinkpad), seek_event)) {
+        /* seek worked, we're done, loop will exit */
+	mpeg2dec->segment_start = GST_EVENT_SEEK_OFFSET (event);
+        res = TRUE;
+	break;
+      }
+    }
+    peer_formats++;
+  }
+  /* at this point, either the seek worked and res = TRUE or res == FALSE and the seek
+   * failed */
+  if (res && flush) {
+    /* if we need to flush, iterate until the buffer is empty */
+    gst_mpeg2dec_flush_decoder (mpeg2dec);
+  }
+
+  return res;
+}
 static gboolean 
 gst_mpeg2dec_src_event (GstPad *pad, GstEvent *event)
 {
@@ -748,65 +1003,26 @@ gst_mpeg2dec_src_event (GstPad *pad, GstEvent *event)
 
   mpeg2dec = GST_MPEG2DEC (gst_pad_get_parent (pad));
 
+  if (mpeg2dec->decoder == NULL)
+    return FALSE;
+
   switch (GST_EVENT_TYPE (event)) {
     /* the all-formats seek logic */
     case GST_EVENT_SEEK:
-    {
-      gint64 src_offset;
-      gboolean flush;
-      GstFormat format;
-      const GstFormat *peer_formats;
-			                
-      format = GST_FORMAT_TIME;
-
-      /* first bring the src_format to TIME */
-      if (!gst_pad_convert (pad,
-                GST_EVENT_SEEK_FORMAT (event), GST_EVENT_SEEK_OFFSET (event),
-                &format, &src_offset))
-      {
-        /* didn't work, probably unsupported seek format then */
-        res = FALSE;
-        break;
+      if (mpeg2dec->need_sequence) {
+        mpeg2dec->pending_event = event;
+	return TRUE;
       }
-
-      /* shave off the flush flag, we'll need it later */
-      flush = GST_EVENT_SEEK_FLAGS (event) & GST_SEEK_FLAG_FLUSH;
-
-      /* assume the worst */
-      res = FALSE;
-
-      /* get our peer formats */
-      peer_formats = gst_pad_get_formats (GST_PAD_PEER (mpeg2dec->sinkpad));
-
-      /* while we did not exhaust our seek formats without result */
-      while (peer_formats && *peer_formats) {
-        gint64 desired_offset;
-
-        format = *peer_formats;
-
-        /* try to convert requested format to one we can seek with on the sinkpad */
-        if (gst_pad_convert (mpeg2dec->sinkpad, GST_FORMAT_TIME, src_offset, &format, &desired_offset))
-        {
-          GstEvent *seek_event;
-
-          /* conversion succeeded, create the seek */
-          seek_event = gst_event_new_seek (format | GST_SEEK_METHOD_SET | flush, desired_offset);
-          /* do the seekk */
-          if (gst_pad_send_event (GST_PAD_PEER (mpeg2dec->sinkpad), seek_event)) {
-            /* seek worked, we're done, loop will exit */
-            res = TRUE;
-          }
-        }
-	peer_formats++;
-      }
-      /* at this point, either the seek worked and res = TRUE or res == FALSE and the seek
-       * failed */
-      if (res && flush) {
-        /* if we need to flush, iterate until the buffer is empty */
-        while (mpeg2_parse (mpeg2dec->decoder) != -1);
+      else {
+        if (mpeg2dec->index)
+          res = index_seek (pad, event);
+	else
+          res = normal_seek (pad, event);
+	
+	if (res)
+          mpeg2dec->discont_state = MPEG2DEC_DISC_NEW_PICTURE;
       }
       break;
-    }
     default:
       res = FALSE;
       break;
@@ -827,16 +1043,18 @@ gst_mpeg2dec_change_state (GstElement *element)
     {
       mpeg2dec->next_time = 0;
       mpeg2dec->peerpool = NULL;
-      mpeg2dec->closed = FALSE;
 
       /* reset the initial video state */
       mpeg2dec->format = MPEG2DEC_FORMAT_NONE;
       mpeg2dec->width = -1;
       mpeg2dec->height = -1;
-      mpeg2dec->first = TRUE;
-      mpeg2dec->last_PTS = -1;
-      mpeg2dec->discont_pending = TRUE;
+      mpeg2dec->segment_start = 0;
+      mpeg2dec->segment_end = -1;
+      mpeg2dec->discont_state = MPEG2DEC_DISC_NEW_PICTURE;
       mpeg2dec->frame_period = 0;
+      mpeg2dec->streaminfo = NULL;
+      gst_mpeg2dec_open_decoder (mpeg2dec);
+      mpeg2dec->need_sequence = TRUE;
       break;
     }
     case GST_STATE_PAUSED_TO_PLAYING:
@@ -853,12 +1071,7 @@ gst_mpeg2dec_change_state (GstElement *element)
       }
       break;
     case GST_STATE_PAUSED_TO_READY:
-      /* if we are not closed by an EOS event do so now, this cen send a few frames but
-       * we are prepared to not really send them (see above) */
-      if (!mpeg2dec->closed) {
-        /*mpeg2_close (mpeg2dec->decoder); */
-	mpeg2dec->closed = TRUE;
-      }
+      gst_mpeg2dec_close_decoder (mpeg2dec);
       break;
     case GST_STATE_READY_TO_NULL:
       break;
@@ -866,9 +1079,7 @@ gst_mpeg2dec_change_state (GstElement *element)
       break;
   }
 
-  GST_ELEMENT_CLASS (parent_class)->change_state (element);
-
-  return GST_STATE_SUCCESS;
+  return GST_ELEMENT_CLASS (parent_class)->change_state (element);
 }
 
 static void
@@ -896,7 +1107,8 @@ gst_mpeg2dec_get_property (GObject *object, guint prop_id, GValue *value, GParam
   mpeg2dec = GST_MPEG2DEC (object);
 
   switch (prop_id) {
-    case ARG_FRAME_RATE:
+    case ARG_STREAMINFO:
+      g_value_set_boxed (value, mpeg2dec->streaminfo);
       break;
     default:
       break;
@@ -916,6 +1128,7 @@ plugin_init (GModule *module, GstPlugin *plugin)
 
   gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (src_template_factory));
   gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (sink_template_factory));
+  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (user_data_template_factory));
 
   gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
 
