@@ -211,11 +211,14 @@ gst_qtdemux_init (GstQTDemux *qtdemux)
 }
 
 static GstCaps*
-quicktime_type_find (GstBuffer *buf,
-	      gpointer private)
+quicktime_type_find (GstBuffer *buf, gpointer private)
 {
   gchar *data = GST_BUFFER_DATA (buf);
 
+  /* we could get a NULL buffer, for example when the input could not be
+   * mmap'd */
+  g_return_val_if_fail (data != NULL, NULL);
+  
   /* exactly like in the old version */
   if (!strncmp (&data[4], "wide", 4) ||
       !strncmp (&data[4], "moov", 4) ||
@@ -687,7 +690,7 @@ gst_qtp_traverse(gpointer poffs,gpointer value,gpointer data)
 
   if (qtdemux->bs_pos < sample->offset) {
     gst_qtp_skip(qtdemux,sample->offset - qtdemux->bs_pos);
-    if (sample->track->pad && GST_PAD_IS_CONNECTED(sample->track->pad)) {
+    if (sample->track->pad && GST_PAD_IS_LINKED(sample->track->pad)) {
       GstBuffer * buf;
       buf = gst_qtp_read(qtdemux,sample->size);
       GST_BUFFER_TIMESTAMP(buf) = sample->timestamp;
