@@ -22,24 +22,28 @@
 
 #include <gstwavparse.h>
 
-static void		gst_wavparse_class_init	(GstWavParseClass *klass);
-static void		gst_wavparse_init	(GstWavParse *wavparse);
+static void		gst_wavparse_class_init		(GstWavParseClass *klass);
+static void		gst_wavparse_init		(GstWavParse *wavparse);
 
-static GstCaps*		wav_type_find		(GstBuffer *buf, gpointer private);
+static GstCaps*		wav_type_find			(GstBuffer *buf, gpointer private);
 
 static const GstFormat*	gst_wavparse_get_formats	(GstPad *pad);
-static const GstPadQueryType *
+static const GstQueryType *
 			gst_wavparse_get_query_types	(GstPad *pad);
-static gboolean		gst_wavparse_pad_query	(GstPad *pad, 
-		                                 GstPadQueryType type,
-						 GstFormat *format, 
-						 gint64 *value);
-static gboolean		gst_wavparse_pad_convert (GstPad *pad,
-						  GstFormat src_format,
-						  gint64 src_value,
-						  GstFormat *dest_format,
-						  gint64 *dest_value);
-static void		gst_wavparse_chain	(GstPad *pad, GstBuffer *buf);
+static gboolean		gst_wavparse_pad_query		(GstPad *pad, 
+		                                	 GstQueryType type,
+							 GstFormat *format, 
+							 gint64 *value);
+static gboolean		gst_wavparse_pad_convert 	(GstPad *pad,
+							 GstFormat src_format,
+							 gint64 src_value,
+							 GstFormat *dest_format,
+							 gint64 *dest_value);
+static void		gst_wavparse_chain		(GstPad *pad, GstBuffer *buf);
+
+static const GstEventMask*
+			gst_wavparse_get_event_masks 	(GstPad *pad);
+static gboolean 	gst_wavparse_srcpad_event 	(GstPad *pad, GstEvent *event);
 
 /* elementfactory information */
 static GstElementDetails gst_wavparse_details = {
@@ -57,8 +61,8 @@ GST_PAD_TEMPLATE_FACTORY (sink_template_factory,
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
-    "wavparse_wav",   
-    "audio/x-wav",  
+    "wavparse_wav",
+    "audio/x-wav",
     NULL
   )
 )
@@ -68,14 +72,14 @@ GST_PAD_TEMPLATE_FACTORY (src_template_factory,
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
-    "wavparse_raw",   
-    "audio/raw",  
+    "wavparse_raw",
+    "audio/raw",
       "format",            GST_PROPS_STRING ("int"),
        "law",              GST_PROPS_INT (0),
-       "endianness",       GST_PROPS_INT (G_BYTE_ORDER),
+       "endianness",       GST_PROPS_INT (G_LITTLE_ENDIAN),
        "signed",           GST_PROPS_LIST (
-       			     GST_PROPS_BOOLEAN (FALSE),
-       			     GST_PROPS_BOOLEAN (TRUE)
+				GST_PROPS_BOOLEAN (FALSE),
+				GST_PROPS_BOOLEAN (TRUE)
 			   ),
        "width",            GST_PROPS_LIST (
 	                     GST_PROPS_INT (8),
@@ -85,14 +89,30 @@ GST_PAD_TEMPLATE_FACTORY (src_template_factory,
 	                     GST_PROPS_INT (8),
 	                     GST_PROPS_INT (16)
 			   ),
-       "rate",             GST_PROPS_INT_RANGE (8000, 48000), 
+       "rate",             GST_PROPS_INT_RANGE (8000, 48000),
        "channels",         GST_PROPS_INT_RANGE (1, 2)
+  ),
+  GST_CAPS_NEW (
+    "wavparse_mp3",
+    "audio/x-mp3",
+    NULL
+  ),
+  GST_CAPS_NEW (
+    "parsewav_law",
+    "audio/raw",
+      "format",            GST_PROPS_STRING ("int"),
+      "law",               GST_PROPS_INT_RANGE (1, 2),
+      "endianness",        GST_PROPS_INT (G_LITTLE_ENDIAN),
+      "width",             GST_PROPS_INT (8),
+      "depth",             GST_PROPS_INT (8),
+      "rate",              GST_PROPS_INT_RANGE (8000, 48000),
+      "channels",          GST_PROPS_INT_RANGE (1, 2)
   )
 )
 
 /* typefactory for 'wav' */
-static GstTypeDefinition 
-wavdefinition = 
+static GstTypeDefinition
+wavdefinition =
 {
   "wavparse_audio/x-wav",
   "audio/x-wav",
@@ -164,9 +184,11 @@ gst_wavparse_init (GstWavParse *wavparse)
   gst_element_add_pad (GST_ELEMENT (wavparse), wavparse->srcpad);
   gst_pad_set_formats_function (wavparse->srcpad, gst_wavparse_get_formats);
   gst_pad_set_convert_function (wavparse->srcpad, gst_wavparse_pad_convert);
-  gst_pad_set_query_type_function (wavparse->srcpad, 
+  gst_pad_set_query_type_function (wavparse->srcpad,
 		                   gst_wavparse_get_query_types);
   gst_pad_set_query_function (wavparse->srcpad, gst_wavparse_pad_query);
+  gst_pad_set_event_function (wavparse->srcpad, gst_wavparse_srcpad_event);
+  gst_pad_set_event_mask_function (wavparse->srcpad, gst_wavparse_get_event_masks);
 
   gst_pad_set_chain_function (wavparse->sinkpad, gst_wavparse_chain);
 
@@ -178,6 +200,8 @@ gst_wavparse_init (GstWavParse *wavparse)
   wavparse->riff_nextlikely = 0;
   wavparse->size = 0;
   wavparse->bps = 0;
+  wavparse->offset = 0;
+  wavparse->need_discont = FALSE;
 }
 
 static GstCaps*
@@ -185,19 +209,18 @@ wav_type_find (GstBuffer *buf, gpointer private)
 {
   gchar *data = GST_BUFFER_DATA (buf);
 
+  if (GST_BUFFER_SIZE (buf) < 12) return NULL;
   if (strncmp (&data[0], "RIFF", 4)) return NULL;
   if (strncmp (&data[8], "WAVE", 4)) return NULL;
 
   return gst_caps_new ("wav_type_find", "audio/x-wav", NULL);
 }
 
-
 static void
 gst_wavparse_chain (GstPad *pad, GstBuffer *buf)
 {
   GstWavParse *wavparse;
   gboolean buffer_riffed = FALSE;	/* so we don't parse twice */
-  gchar *data;
   gulong size;
 
   g_return_if_fail (pad != NULL);
@@ -208,27 +231,55 @@ gst_wavparse_chain (GstPad *pad, GstBuffer *buf)
   wavparse = GST_WAVPARSE (gst_pad_get_parent (pad));
   GST_DEBUG (0, "gst_wavparse_chain: got buffer in '%s'",
           gst_object_get_name (GST_OBJECT (wavparse)));
-  data = (guchar *) GST_BUFFER_DATA (buf);
+
   size = GST_BUFFER_SIZE (buf);
 
   /* walk through the states in priority order */
   /* we're in the data region */
   if (wavparse->state == GST_WAVPARSE_DATA) {
+    GstFormat format;
+    guint64 maxsize;
+
+    /* we can't go beyond the max length */
+    maxsize = wavparse->riff_nextlikely - GST_BUFFER_OFFSET (buf);
+
     /* if we're expected to see a new chunk in this buffer */
-    if ((wavparse->riff_nextlikely - GST_BUFFER_OFFSET (buf)) < GST_BUFFER_SIZE (buf)) {
-	    
-      GST_BUFFER_SIZE (buf) = wavparse->riff_nextlikely - GST_BUFFER_OFFSET (buf);
-      
+    if (maxsize < size) {
+      GstBuffer *newbuf;
+
+      newbuf = gst_buffer_create_sub (buf, 0, maxsize);
+      gst_buffer_unref (buf);
+      buf = newbuf;
+
+      size = maxsize;
+
       wavparse->state = GST_WAVPARSE_OTHER;
       /* I suppose we could signal an EOF at this point, but that may be
          premature.  We've stopped data flow, that's the main thing. */
-    } 
+    }
 
-    GST_BUFFER_TIMESTAMP (buf) = wavparse->offset * GST_SECOND / wavparse->rate;
-    wavparse->offset += GST_BUFFER_SIZE (buf) * 8 / wavparse->width / wavparse->channels;
+    if (GST_PAD_IS_USABLE (wavparse->srcpad)) {
+      format = GST_FORMAT_TIME;
+      gst_pad_convert (wavparse->srcpad,
+		       GST_FORMAT_BYTES,
+		       wavparse->offset,
+		       &format,
+		       &GST_BUFFER_TIMESTAMP (buf));
 
-    if (GST_PAD_IS_USABLE (wavparse->srcpad))
+      if (wavparse->need_discont) {
+        gst_pad_push (wavparse->srcpad, 
+	              GST_BUFFER (gst_event_new_discontinuous (FALSE,
+			      GST_FORMAT_TIME, GST_BUFFER_TIMESTAMP (buf), 
+			      NULL)));
+        wavparse->need_discont = FALSE;
+      }
       gst_pad_push (wavparse->srcpad, buf);
+    }
+    else
+      gst_buffer_unref (buf);
+
+    wavparse->offset += size;
+
     return;
   }
 
@@ -250,7 +301,7 @@ gst_wavparse_chain (GstPad *pad, GstBuffer *buf)
 
     /* create a new RIFF parser */
     wavparse->riff = gst_riff_new ();
-    
+
     /* give it the current buffer to start parsing */
     retval = gst_riff_next_buffer (wavparse->riff, buf, 0);
     buffer_riffed = TRUE;
@@ -287,8 +338,7 @@ gst_wavparse_chain (GstPad *pad, GstBuffer *buf)
 
     /* if we've got something, deal with it */
     if (fmt != NULL) {
-      GstCaps *caps;
-
+      GstCaps *caps = NULL;
 
       /* we can gather format information now */
       format = (GstWavParseFormat *)((guchar *) GST_BUFFER_DATA (buf) + fmt->offset);
@@ -297,20 +347,56 @@ gst_wavparse_chain (GstPad *pad, GstBuffer *buf)
       wavparse->rate     = GUINT32_FROM_LE(format->dwSamplesPerSec);
       wavparse->channels = GUINT16_FROM_LE(format->wChannels);
       wavparse->width    = GUINT16_FROM_LE(format->wBitsPerSample);
-      
+      wavparse->format	 = GINT16_FROM_LE(format->wFormatTag);
+
       /* set the caps on the src pad */
-      caps = GST_CAPS_NEW (
+      /* FIXME: handle all of the other formats as well */
+      switch (wavparse->format)
+      {
+        case GST_RIFF_WAVE_FORMAT_ALAW:
+        case GST_RIFF_WAVE_FORMAT_MULAW:
+	  if (!(wavparse->width == 8)) {
+	    gst_element_error (GST_ELEMENT (wavparse), "wavparse: invalid wave file");
+	    return;
+	  }
+          caps = GST_CAPS_NEW (
 			"parsewav_src",
 			"audio/raw",
 			"format",	GST_PROPS_STRING ("int"),
-			  "law",	GST_PROPS_INT (0),		/*FIXME */
-			  "endianness",	GST_PROPS_INT (G_BYTE_ORDER),
-        		  "signed",     GST_PROPS_BOOLEAN ((wavparse->width > 8) ? TRUE : FALSE),
+			  "law",	GST_PROPS_INT (wavparse->format == GST_RIFF_WAVE_FORMAT_ALAW ? 2 : 1),
+			  "endianness",	GST_PROPS_INT (G_LITTLE_ENDIAN),
+			  "width",	GST_PROPS_INT (8),
+			  "depth",	GST_PROPS_INT (8),
+			  "rate",	GST_PROPS_INT (wavparse->rate),
+			  "channels",	GST_PROPS_INT (wavparse->channels)
+		);
+	  break;
+        case GST_RIFF_WAVE_FORMAT_PCM:
+          caps = GST_CAPS_NEW (
+			"parsewav_src",
+			"audio/raw",
+			"format",	GST_PROPS_STRING ("int"),
+			  "law",	GST_PROPS_INT (0),
+			  "endianness",	GST_PROPS_INT (G_LITTLE_ENDIAN),
+			  "signed",     GST_PROPS_BOOLEAN ((wavparse->width > 8) ? TRUE : FALSE),
 			  "width",	GST_PROPS_INT (wavparse->width),
 			  "depth",	GST_PROPS_INT (wavparse->width),
 			  "rate",	GST_PROPS_INT (wavparse->rate),
 			  "channels",	GST_PROPS_INT (wavparse->channels)
-		      );
+		);
+	  break;
+        case GST_RIFF_WAVE_FORMAT_MPEGL12:
+        case GST_RIFF_WAVE_FORMAT_MPEGL3:
+	  caps = GST_CAPS_NEW (
+			"parsewav_src",
+			"audio/x-mp3",
+			NULL
+		);
+	  break;
+	default:
+          gst_element_error (GST_ELEMENT (wavparse), "wavparse: format %d not handled", wavparse->format);
+          return;
+      }
 
       if (gst_pad_try_set_caps (wavparse->srcpad, caps) <= 0) {
         gst_element_error (GST_ELEMENT (wavparse), "Could not set caps");
@@ -318,7 +404,7 @@ gst_wavparse_chain (GstPad *pad, GstBuffer *buf)
       }
 
       GST_DEBUG (0, "frequency %d, channels %d",
-		 wavparse->rate, wavparse->channels); 
+		 wavparse->rate, wavparse->channels);
 
       /* we're now looking for the data chunk */
       wavparse->state = GST_WAVPARSE_CHUNK_DATA;
@@ -349,25 +435,26 @@ gst_wavparse_chain (GstPad *pad, GstBuffer *buf)
 
       GST_DEBUG (0, "data begins at %ld", datachunk->offset);
 
+      wavparse->datastart = datachunk->offset;
+
       /* at this point we can ACK that we have data */
       wavparse->state = GST_WAVPARSE_DATA;
 
       /* now we construct a new buffer for the remainder */
       subsize = size - datachunk->offset;
       GST_DEBUG (0, "sending last %ld bytes along as audio", subsize);
-      
-      newbuf = gst_buffer_new ();
-      GST_BUFFER_DATA (newbuf) = g_malloc (subsize);
-      GST_BUFFER_SIZE (newbuf) = subsize;
-      GST_BUFFER_TIMESTAMP (newbuf) = wavparse->offset * GST_SECOND / wavparse->rate;
-      wavparse->offset += subsize * 8 / wavparse->width / wavparse->channels;
-      
-      memcpy (GST_BUFFER_DATA (newbuf), GST_BUFFER_DATA (buf) + datachunk->offset, subsize);
 
+      newbuf = gst_buffer_create_sub (buf, datachunk->offset, subsize);
       gst_buffer_unref (buf);
 
+      GST_BUFFER_TIMESTAMP (newbuf) = 0;
+
       if (GST_PAD_IS_USABLE (wavparse->srcpad))
         gst_pad_push (wavparse->srcpad, newbuf);
+      else
+	gst_buffer_unref (newbuf);
+
+      wavparse->offset = subsize;
 
       /* now we're ready to go, the next buffer should start data */
       wavparse->state = GST_WAVPARSE_DATA;
@@ -406,22 +493,23 @@ gst_wavparse_pad_convert (GstPad *pad,
   GstWavParse *wavparse;
 
   wavparse = GST_WAVPARSE (gst_pad_get_parent (pad));
+  /* FIXME default should be samples in this case IMO */
   if (*dest_format == GST_FORMAT_DEFAULT)
     *dest_format = GST_FORMAT_TIME;
   
   bytes_per_sample = wavparse->channels * wavparse->width / 8;
   if (bytes_per_sample == 0) {
-	  g_warning ("bytes_per_sample is 0, internal error\n");
-	  g_warning ("channels %d,  width %d\n",
-		     wavparse->channels, wavparse->width);
-	  return FALSE;
+    g_warning ("bytes_per_sample is 0, internal error\n");
+    g_warning ("channels %d,  width %d\n",
+	     wavparse->channels, wavparse->width);
+    return FALSE;
   }
   byterate = (glong) (bytes_per_sample * wavparse->rate);
   if (byterate == 0) {
-	  g_warning ("byterate is 0, internal error\n");
-	  return FALSE;
+    g_warning ("byterate is 0, internal error\n");
+    return FALSE;
   }
-  g_print ("DEBUG: bytes per sample: %d\n", bytes_per_sample);
+  GST_DEBUG (0, "bytes per sample: %d\n", bytes_per_sample);
 
   switch (src_format) {
     case GST_FORMAT_BYTES:
@@ -444,9 +532,11 @@ gst_wavparse_pad_convert (GstPad *pad,
       if (*dest_format == GST_FORMAT_BYTES)
 	*dest_value = src_value * byterate / GST_SECOND;
       else if (*dest_format == GST_FORMAT_UNITS)
-        *dest_value = src_value * wavparse->rate /GST_SECOND;
+	*dest_value = src_value * wavparse->rate / GST_SECOND;
       else
         return FALSE;
+
+      *dest_value = *dest_value & ~(bytes_per_sample - 1);
       break;
     default:
       g_warning ("unhandled format for wavparse\n");
@@ -455,19 +545,20 @@ gst_wavparse_pad_convert (GstPad *pad,
   return TRUE;
 }
       
-static const GstPadQueryType *
+static const GstQueryType *
 gst_wavparse_get_query_types (GstPad *pad)
 {
-  static const GstPadQueryType types[] = {
-    GST_PAD_QUERY_TOTAL,
-    GST_PAD_QUERY_POSITION
+  static const GstQueryType types[] = {
+    GST_QUERY_TOTAL,
+    GST_QUERY_POSITION,
+    0
   };
   return types;
 }
 
 /* handle queries for location and length in requested format */
 static gboolean
-gst_wavparse_pad_query (GstPad *pad, GstPadQueryType type,
+gst_wavparse_pad_query (GstPad *pad, GstQueryType type,
 			GstFormat *format, gint64 *value)
 {
   GstFormat peer_format = GST_FORMAT_BYTES;
@@ -488,10 +579,76 @@ gst_wavparse_pad_query (GstPad *pad, GstPadQueryType type,
     g_warning ("Could not query sink pad's peer\n");
     return FALSE;
   }
-  g_print ("DEBUG: pad_query done, value %lld\n", *value);
+  GST_DEBUG (0, "pad_query done, value %" G_GINT64_FORMAT "\n", *value);
   return TRUE;
 }
 
+static const GstEventMask*
+gst_wavparse_get_event_masks (GstPad *pad)
+{ 
+  static const GstEventMask gst_wavparse_src_event_masks[] = {
+    { GST_EVENT_SEEK, GST_SEEK_METHOD_SET |
+                      GST_SEEK_FLAG_FLUSH },
+    { 0, }
+  };
+  return gst_wavparse_src_event_masks;
+}   
+
+static gboolean
+gst_wavparse_srcpad_event (GstPad *pad, GstEvent *event)
+{
+  GstWavParse *wavparse = GST_WAVPARSE (GST_PAD_PARENT (pad));
+  gboolean res = FALSE;
+
+  GST_DEBUG(0, "event %d", GST_EVENT_TYPE (event));
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_SEEK:
+    {
+      gint64 byteoffset;
+      GstFormat format;
+
+      /* we can only seek when in the DATA state */
+      if (wavparse->state != GST_WAVPARSE_DATA) {
+	return FALSE;
+      }
+
+      format = GST_FORMAT_BYTES;
+      
+      /* bring format to bytes for the peer element, 
+       * FIXME be smarter here */
+      res = gst_pad_convert (pad, 
+		             GST_EVENT_SEEK_FORMAT (event),
+		             GST_EVENT_SEEK_OFFSET (event),
+		             &format,
+		             &byteoffset);
+      
+      if (res) {
+	 GstEvent *seek;
+
+	 /* seek to byteoffset + header length */
+	 seek = gst_event_new_seek (
+			 GST_FORMAT_BYTES |
+			 (GST_EVENT_SEEK_TYPE (event) & ~GST_SEEK_FORMAT_MASK), 
+			 byteoffset + wavparse->datastart);
+
+	 res = gst_pad_send_event (GST_PAD_PEER (wavparse->sinkpad), seek);
+
+	 if (res) {
+           /* ok, seek worked, update our state */
+           wavparse->offset = byteoffset; 
+	   wavparse->need_discont = TRUE;
+	 }
+      }
+      break;
+    }
+    default:
+      break;
+  }
+
+  gst_event_unref (event);
+  return res;
+}
 
 static gboolean
 plugin_init (GModule *module, GstPlugin *plugin)
