@@ -168,15 +168,25 @@ _gst_ximage_destroy (GstXImage *image)
 static void
 _gst_ximage_put (GstXWindow *window, GstXImage *image)
 {
- if (image->SHMInfo.shmaddr) {
-   XShmPutImage(window->disp, window->win, 
-		window->gc, image->ximage, 
-		0, 0, 0, 0, image->width, image->height, 
-		False);
- } else {
-   XPutImage(window->disp, window->win, 
-	     window->gc, image->ximage,  
-	     0, 0, 0, 0, image->width, image->height);
- }
- XSync(window->disp, False);
+  XWindowAttributes attr; 
+  int x, y;
+
+  g_return_if_fail (window != NULL);
+  g_return_if_fail (image != NULL);
+
+  XGetWindowAttributes(window->disp, window->win, &attr); 
+
+  x = MAX (0, (attr.width - image->width) / 2);
+  y = MAX (0, (attr.height - image->height) / 2);
+  if (image->SHMInfo.shmaddr) {
+    XShmPutImage(window->disp, window->win, 
+		 window->gc, image->ximage, 
+		 0, 0, x, y, image->width, image->height, 
+		 False);
+  } else {
+    XPutImage(window->disp, window->win, 
+	      window->gc, image->ximage,  
+	      0, 0, x, y, image->width, image->height);
+  }
+  XSync(window->disp, False);
 }
