@@ -76,8 +76,10 @@ static void		gst_xvideosink_get_property	(GObject *object, guint prop_id,
 
 static GstCaps* 	gst_xvideosink_get_pad_template_caps (gboolean with_xv);
 
-static GstPadTemplate *sink_template;
-static GstCaps *formats;
+/* default template - initiated with class struct to allow gst-register to work
+   with X running */
+static GstPadTemplate *sink_template = NULL;
+static GstCaps *formats = NULL;
 
 static GstElementClass *parent_class = NULL;
 static guint gst_xvideosink_signals[LAST_SIGNAL] = { 0 };
@@ -161,7 +163,15 @@ gst_xvideosink_class_init (GstXVideoSinkClass *klass)
   gstelement_class->change_state = gst_xvideosink_change_state;
   gstelement_class->set_clock = gst_xvideosink_set_clock;
   gstelement_class->release_locks = gst_xvideosink_release_locks;
-
+  
+  /* initiate default template */
+  formats = gst_xvideosink_get_pad_template_caps (TRUE);
+  sink_template = gst_pad_template_new (
+                      "sink",
+                      GST_PAD_SINK,
+                      GST_PAD_ALWAYS,
+                      formats,
+                      NULL);
 }
 
 static void
@@ -425,7 +435,7 @@ gst_xvideosink_getcaps (GstPad *pad, GstCaps *caps)
 
   xvideosink = GST_XVIDEOSINK (gst_pad_get_parent (pad));
 
-  return xvideosink->formats;
+  return gst_caps_ref (xvideosink->formats);
 }
 
 static void
@@ -438,6 +448,7 @@ gst_xvideosink_init (GstXVideoSink *xvideosink)
   gst_pad_set_getcaps_function (xvideosink->sinkpad, gst_xvideosink_getcaps);
   gst_pad_set_bufferpool_function (xvideosink->sinkpad, gst_xvideosink_get_bufferpool);
 
+  xvideosink->window = NULL;
   xvideosink->toplevel = TRUE;
   xvideosink->width = 100;
   xvideosink->height = 100;
@@ -617,8 +628,8 @@ gst_xvideosink_set_property (GObject *object, guint prop_id,
     case ARG_NEED_NEW_WINDOW:
       if (g_value_get_boolean (value)) {
         g_mutex_lock (xvideosink->lock);
-        /* FIXME destroying the window causes a crash, so we'll let it leak for now */
-        /*_gst_xwindow_destroy (xvideosink->window);*/
+        /*if (xvideosink->window)
+          _gst_xwindow_destroy (xvideosink->window);*/
         xvideosink->window = _gst_xwindow_new (xvideosink->width, 
 			                       xvideosink->height, 
 					       xvideosink->toplevel);
@@ -799,15 +810,15 @@ plugin_init (GModule *module, GstPlugin *plugin)
                                    &gst_xvideosink_details);
   g_return_val_if_fail(factory != NULL, FALSE);
 
-  formats = gst_xvideosink_get_pad_template_caps (TRUE);
-
-  sink_template = gst_pad_template_new (
-		  "sink",
+  gst_element_factory_add_pad_template (factory, 
+          gst_pad_template_new (
+                  "sink",
                   GST_PAD_SINK,
-  		  GST_PAD_ALWAYS,
-		  formats, NULL);
-
-  gst_element_factory_add_pad_template (factory, sink_template);
+                  GST_PAD_ALWAYS,
+                  gst_caps_new ("video_caps",
+                                "video/raw",
+                                NULL),
+                  NULL));
 
   gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
 
